package cn.bunny.service.web.aop.aspect;

import cn.bunny.common.service.utils.JwtHelper;
import cn.bunny.entity.system.log.SystemLog;
import cn.bunny.service.web.aop.annotation.SkipLog;
import cn.bunny.service.web.mapper.SystemLogMapper;
import cn.bunny.vo.system.login.LoginVo;
import com.alibaba.fastjson2.JSONObject;
import jakarta.servlet.http.HttpServletRequest;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import java.util.Arrays;
import java.util.Map;

@Aspect
@Component
@Slf4j
public class AutoLogAspect {
    @Autowired
    private SystemLogMapper systemLogMapper;

    @Pointcut("execution(* cn.bunny.service.web.controller..*(..))")
    public void point() {
    }

    @Around(value = "point()")
    public Object aroundMethod(ProceedingJoinPoint joinPoint) throws Throwable {
        Object result;
        MethodSignature signature = (MethodSignature) joinPoint.getSignature();
        // 是否有跳过注解，如果有跳过注解就不执行当前操作！！！
        SkipLog annotation = signature.getMethod().getAnnotation(SkipLog.class);
        // 目标方法所在类名路径
        String classPath = joinPoint.getSignature().getDeclaringTypeName();
        // 当前执行的方法名
        String methodName = signature.getName();
        // 入参内容
        String args = Arrays.toString(joinPoint.getArgs());
        // 获取用户token
        ServletRequestAttributes requestAttributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = requestAttributes.getRequest();
        String token = request.getHeader("token");
        // 初始化系统日志对象
        SystemLog systemLog = new SystemLog();
        // token转为实体对象
        Map<String, Object> mapByToken = JwtHelper.getMapByToken(token);
        LoginVo loginVo = JSONObject.parseObject(JSONObject.toJSONString(mapByToken), LoginVo.class);

        // 插入Ip地址
        systemLog.setIpAddress(request.getRemoteHost());

        try {
            // 当为null时跳过执行
            if (annotation != null) return joinPoint.proceed();
            if (args.equals("[null]")) {
                systemLog.setArgs(null);
            } else {
                systemLog.setArgs(args);
            }
            // 登录返回Vo不为空即插入
            if (loginVo != null) {
                systemLog.setNickname(loginVo.getNickName());
                systemLog.setEmail(loginVo.getEmail());
                systemLog.setUpdateUser(loginVo.getId());
            }

            systemLog.setClassPath(classPath);
            systemLog.setMethodName(methodName);
            systemLog.setToken(token);

            // 目标对象（连接点）方法的执行
            result = joinPoint.proceed();
            systemLog.setResult(JSONObject.toJSONString(result));
        } catch (Exception exception) {
            String message = exception.getMessage();
            StackTraceElement[] stackTrace = exception.getStackTrace();

            // 如果报错，设置报错的堆栈和消息，放到数据库中
            systemLog.setErrorStack(Arrays.toString(stackTrace));
            systemLog.setErrorMessage(message);

            // 插入日志数据到数据库
            systemLogMapper.insert(systemLog);

            throw exception;
        }

        // 插入日志数据到数据库
        systemLogMapper.insert(systemLog);
        return result;
    }
}
