package cn.bunny.service.web.service.impl;

import cn.bunny.common.service.context.BaseContext;
import cn.bunny.common.service.exception.BunnyException;
import cn.bunny.common.service.utils.EmptyUtil;
import cn.bunny.common.service.utils.JwtHelper;
import cn.bunny.dto.user.LoginDto;
import cn.bunny.dto.user.RegisterDto;
import cn.bunny.pojo.email.EmailSend;
import cn.bunny.pojo.email.EmailSendInit;
import cn.bunny.entity.system.email.EmailTemplate;
import cn.bunny.entity.system.email.EmailUsers;
import cn.bunny.entity.system.user.User;
import cn.bunny.module.mail.utils.MailSenderUtil;
import cn.bunny.pojo.result.constant.ExceptionConstant;
import cn.bunny.pojo.result.constant.MailMessageConstant;
import cn.bunny.pojo.result.constant.RedisUserConstant;
import cn.bunny.pojo.result.constant.UserConstant;
import cn.bunny.service.web.mapper.EmailTemplateMapper;
import cn.bunny.service.web.mapper.EmailUsersMapper;
import cn.bunny.service.web.mapper.UserMapper;
import cn.bunny.service.web.service.LoginService;
import cn.bunny.vo.system.login.LoginVo;
import cn.bunny.vo.system.login.ValidateCodeVo;
import cn.hutool.captcha.CaptchaUtil;
import cn.hutool.captcha.CircleCaptcha;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.util.DigestUtils;

import java.util.UUID;
import java.util.concurrent.TimeUnit;

@Service
public class LoginServiceImpl extends ServiceImpl<UserMapper, User> implements LoginService {
    @Autowired
    private UserMapper userMapper;
    @Autowired
    private RedisTemplate<String, Object> redisTemplate;
    @Autowired
    private EmailUsersMapper emailUsersMapper;
    @Autowired
    private EmailTemplateMapper emailTemplateMapper;

    /**
     * 前台用户登录接口
     *
     * @param loginDto 登录参数
     * @return 登录后结果返回
     */
    @Override
    public LoginVo login(LoginDto loginDto) {
        // 判断用户和密码是否为空
        String username = loginDto.getUsername();
        EmptyUtil.isEmpty(username, ExceptionConstant.USERNAME_IS_EMPTY_Exception);
        // 密码不能为空
        EmptyUtil.isEmpty(loginDto.getPassword(), ExceptionConstant.PASSWORD_NOT_EMPTY_Exception);
        String password = DigestUtils.md5DigestAsHex(loginDto.getPassword().getBytes());

        // 设置返回类型
        LoginVo loginVo = userMapper.login(username, password);
        String token = JwtHelper.createToken(loginVo.getId(), loginVo.getEmail(), 7);
        loginVo.setToken(token);

        // 用户是否被锁定
        if (loginVo.getStatus() == 1) {
            throw new BunnyException(ExceptionConstant.ACCOUNT_LOCKED_Exception);
        }

        // 将信息存入Redis中
        redisTemplate.opsForValue().set(RedisUserConstant.getUserLoginInfoPrefix(loginVo.getEmail()), loginVo, 15, TimeUnit.DAYS);
        return loginVo;
    }

    /**
     * 生成验证码
     *
     * @return 验证码生成信息
     */
    @Override
    public ValidateCodeVo checkCode() {
        // 生成验证码
        CircleCaptcha captcha = CaptchaUtil.createCircleCaptcha(150, 48, 4, 2);
        String code = captcha.getCode();
        String base64Image = "data:image/png;base64," + captcha.getImageBase64();
        // 存储到Redis中
        String uuid = UUID.randomUUID().toString().replaceAll("-", "");
        redisTemplate.opsForValue().set(uuid, code, 10, TimeUnit.MINUTES);
        return ValidateCodeVo.builder().codeKey(uuid).codeValue(base64Image).build();
    }

    /**
     * 发送邮箱验证码
     *
     * @param email 邮箱
     */
    @Override
    public void sendEmailCode(String email) {
        // 从数据库中获取，发送邮箱参数，发送邮件模板
        EmailUsers emailUsers = emailUsersMapper.selectOne(Wrappers.<EmailUsers>lambdaQuery().eq(EmailUsers::getIsDefault, 1));
        EmailTemplate emailSendTemplate = emailTemplateMapper.selectOne(Wrappers.<EmailTemplate>lambdaQuery().eq(EmailTemplate::getId, 1791870020197625858L));
        EmailSendInit emailSendInit = new EmailSendInit();
        // 判断发送邮箱邮件是否为空
        EmptyUtil.isEmpty(emailUsers, MailMessageConstant.EMAIL_CONFIG_NOT_FOUND);
        BeanUtils.copyProperties(emailUsers, emailSendInit);
        emailSendInit.setUsername(emailUsers.getEmail());
        // 生成验证码
        MailSenderUtil mailSenderUtil = new MailSenderUtil(emailSendInit);
        CircleCaptcha captcha = CaptchaUtil.createCircleCaptcha(150, 48, 4, 2);
        // 生成验证码和可以
        String code = captcha.getCode();
        String htmlContent = emailSendTemplate.getBody().replace("${verifyCode}", code);
        // 发送验证码
        EmailSend emailSend = new EmailSend();
        emailSend.setSubject("邮箱验证码");
        emailSend.setMessage(htmlContent);
        emailSend.setSendTo(email);
        emailSend.setIsRichText(true);
        mailSenderUtil.sendSimpleEmail(emailSend);

        // 将验证码保存到Redis中，并设置15分钟过期
        redisTemplate.opsForValue().set(RedisUserConstant.USER_EMAIL_CODE_PREFIX + email, code, 15, TimeUnit.MINUTES);
    }

    /**
     * 注册用户
     *
     * @param registerDto 注册表单
     */
    @Override
    public void register(RegisterDto registerDto) {
        String nickName = registerDto.getNickName();
        String email = registerDto.getEmail();
        // 查询用户是否已经注册
        LambdaQueryWrapper<User> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(User::getEmail, email).or().eq(User::getNickName, nickName);
        User user = userMapper.selectOne(lambdaQueryWrapper);

        // 用户存在保存
        if (user != null) throw new BunnyException(ExceptionConstant.ALREADY_USER_Exception);
        // 判断验证码是否正确
        Object emailCode = redisTemplate.opsForValue().get(RedisUserConstant.getUserEmailCodePrefix(email));
        if (!registerDto.getCode().equalsIgnoreCase(String.valueOf(emailCode))) {
            throw new BunnyException(ExceptionConstant.VERIFICATION_CODE_ERROR_Exception);
        }

        // 用户没有注册时才去注册
        user = new User();
        BeanUtils.copyProperties(registerDto, user);
        user.setPassword(DigestUtils.md5DigestAsHex(registerDto.getPassword().getBytes()));
        user.setAvatar(UserConstant.USER_AVATAR);
        save(user);

        // 删除邮箱验证码
        redisTemplate.delete(RedisUserConstant.getUserEmailCodePrefix(registerDto.getEmail()));
    }

    /**
     * 退出登录
     */
    @Override
    public void logout() {
        Long userId = BaseContext.getUserId();
        // 查询当前用户信息
        User user = getById(userId);
        // 删除Redis中用户信息
        redisTemplate.delete(RedisUserConstant.getUserLoginInfoPrefix(user.getEmail()));
    }
}
